/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package general;

import java.util.HashMap;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.JComponent;

import yacy.Codings;

public class Config {

	public static final int LOG_QUIET = 0;
	public static final int LOG_NORMAL = 1;
	public static final int LOG_DEBUG = 2;
	
	public static final int neededSVNRevision = 3117;
	public String version = null;
	
	public boolean useGZipEncoding = false;
	public boolean useHTTP11 = true;
	public boolean useChunkedTransfer = true;
	
	public boolean logShowAlways = false;
	public int logHideDelay = 5000;
	
	public boolean useGlobalUpdateDelay = false;
	public int globalUpdateDelay = 3000;
	
	public String connectOnStartup = null;
	public boolean noConnectOnStartup = false;
	public boolean noVersionCheck = false;
	public String connectOnStartupOverride = null;
	public String connectOnStartupOverrideUser = null;
	public String connectOnStartupOverridePwd = null;
	public boolean useHTTPS = false;
	public int timeout = 10000;
	
	public HashMap<String,String> connects = null;
	
	public int WindowWidth  = 700;
	public int WindowHeight = 520;
	
	/* command line options */
	public int logLevel = LOG_NORMAL;
	public boolean loadPlugins = true;
	
	public JComponent busyAnimator = null;
	public Log log = null;
	
	public String proxyHost = null;
	public int proxyPort = 0;
	public boolean proxyEnabled = false;
	
	public Config(String version) {
		connects = new HashMap<String,String>();
		this.version = version;
	}
	
	public String 	getVersion()  { return this.version; }
	public boolean 	isProxyEnabled() { return this.proxyEnabled; }
	public String 	getProxyHost() { return this.proxyHost; }
	public int 		getProxyPort() { return this.proxyPort; }
	
	public void log(Throwable e) {
		if (logLevel == Config.LOG_NORMAL)
			System.err.println(e.getMessage());
		else if (logLevel == Config.LOG_DEBUG)
			e.printStackTrace();
	}
	
	public void log(String text, Throwable e) {
		if (logLevel == Config.LOG_NORMAL)
			System.err.println(text + e.getMessage());
		else if (logLevel == Config.LOG_DEBUG)
			e.printStackTrace();
	}
	
	public void log(String text) {
		if (logLevel > Config.LOG_QUIET && text != null)
			System.out.print(text);
	}
	
	public void log(String text, boolean newLine) {
		log(text);
		if (logLevel > Config.LOG_QUIET && newLine)
			System.out.println();
	}
	
	public void logDebugAdditional(String normal, String debug) {
		if (logLevel > Config.LOG_QUIET) {
			System.out.print(normal);
			if (logLevel == Config.LOG_DEBUG)
				System.out.print(debug);
		}
	}
	
	public void logDebugAdditional(String normal, String debug, boolean newLine) {
		logDebugAdditional(normal, debug);
		if (newLine)
			System.out.println();
	}
	
	public void logDebug(String text) {
		if (logLevel == Config.LOG_DEBUG)
			System.out.print(text);
	}
	
	public void logDebug(String text, boolean newLine) {
		logDebug(text);
		if (newLine)
			System.out.println();
	}
	
	public void logDebug(Throwable e) {
		if (logLevel == Config.LOG_DEBUG)
			e.printStackTrace();
	}
	
	public void setProxy(String proxyHost, int proxyPort) {
		this.proxyHost = proxyHost;
		this.proxyPort = proxyPort;
	}
	
	public void setProxyEnabled(boolean yes) {
		this.proxyEnabled = yes;
		updateProxySettings();
	}
	
	private void updateProxySettings() {
		if (proxyHost == null || proxyPort == 0) {
			System.clearProperty("http.proxyHost");
			System.clearProperty("http.proxyPort");
		} else {
			System.setProperty("http.proxyHost", proxyHost);
			System.setProperty("http.proxyPort", Integer.toString(proxyPort));
		}
	}
	
	public void putConnect(String host, int port, String user, String pwd) {
		this.connects.put(user + '@' + host + ':' + port, pwd);
	}
	
	public void putConnectPlain(String host, int port, String user, String pwd) {
		putConnect(host, port, user, Codings.encodeMD5Hex(Codings.encodeString(user + ':' + pwd)));
	}
	
	public String getConnect(String host, int port, String user) {
		return this.connects.get(user + '@' + host + ':' + port);
	}
	
	public void load() {
		Preferences a = Preferences.userRoot().node("yacy-admin").node("connects");
		Preferences b = Preferences.userRoot().node("yacy-admin").node("settings");
		
		this.useGZipEncoding = b.getBoolean("useGZip", false);
		this.useHTTP11 = b.getBoolean("http/1.1", true);
		this.useChunkedTransfer = b.getBoolean("use-chunked-transfer", true);
		
		this.logShowAlways = !b.getBoolean("loghide", true);
		this.logHideDelay = b.getInt("loghide-interval", 3000);
		
		this.useGlobalUpdateDelay = b.getBoolean("update-interval_enabled", false);
		this.globalUpdateDelay = b.getInt("update-interval", 3000);
		
		this.connectOnStartup = b.get("auto-connect", "");
		this.useHTTPS = b.get("connect-type", "http").equals("https");
		this.timeout = b.getInt("connection-timeout", 10000);
		
		this.setProxy(b.get("proxyHost", null), b.getInt("proxyPort", 0));
		this.setProxyEnabled(b.getBoolean("proxyEnabled", false));

		try {
			for (int i = 0; i < a.keys().length; i++)
				connects.put(a.keys()[i], a.get(a.keys()[i], ""));
		} catch (BackingStoreException e) { log(e); }
		
		this.WindowWidth  = b.getInt("WindowWidth", 700);
		this.WindowHeight = b.getInt("WindowHeight", 520);
	}
	
	public void save() {
		Preferences a = Preferences.userRoot().node("yacy-admin").node("connects");
		Preferences b = Preferences.userRoot().node("yacy-admin").node("settings");
		
		b.putBoolean("useGZip", this.useGZipEncoding);
		b.putBoolean("http/1.1", this.useHTTP11);
		b.putBoolean("use-chunked-transfer", this.useChunkedTransfer);
		
		b.putBoolean("loghide", !this.logShowAlways);
		b.putInt("loghide-interval", this.logHideDelay);
		
		b.putBoolean("update-interval_enabled", this.useGlobalUpdateDelay);
		b.putInt("update-interval", this.globalUpdateDelay);
		
		b.put("auto-connect", this.connectOnStartup);
		b.put("connect-type", ((this.useHTTPS) ? ("https") : ("http")));
		b.putInt("connection-timeout", this.timeout);
		
		b.put("proxyHost", (this.proxyHost == null) ? ("") : (this.proxyHost));
		b.putInt("proxyPort", this.proxyPort);
		b.putBoolean("proxyEnabled", this.proxyEnabled);
		
		try {
			a.clear();
		} catch (BackingStoreException e) { log("Error on saving history.", e); }
		
		for (String d : this.connects.keySet())
			a.put(d, this.connects.get(d));
		
		b.putInt("WindowWidth",this.WindowWidth);
		b.putInt("WindowHeight",this.WindowHeight);
	}
}
