/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package plugins;

import general.Config;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import javax.swing.ImageIcon;
import javax.swing.JButton;

import views.ToolbarPlugin;
import yacy.Peer;
import yacy.event.PeerEvent;
import yacy.event.PeerEventListener;
import yacy.event.PeerInterruptableEvent;
import yacy.event.PeerSettingChangedEvent;

public class CrawlPauseResume extends JButton implements ToolbarPlugin {

	static final long serialVersionUID = 1L;
	
	private Config 	cfg 		= null;
	private Peer 	peer 		= null;
	private boolean statePaused = false;
	private boolean notWorking 	= false;
	private EL 		el 			= null;
	
	private static final ImageIcon pause = new ImageIcon("ressources/pause.png");
	private static final ImageIcon start = new ImageIcon("ressources/start.png");
	
	public String 	getVersion() 			{ return "0.1"; }
	public int 		getNeededYacyRevision() { return 2958; }
	
	public CrawlPauseResume() {
		super();
		this.el = new EL();
		initialize();
	}
	
	public void init(Config cfg) {
		this.cfg = cfg;
	}
	
	private class EL implements PeerEventListener {
		public void peerEvent(PeerEvent e) {
			if (e instanceof PeerSettingChangedEvent &&
					((PeerSettingChangedEvent)e).getDescription().indexOf("queue state") > -1) {
				statePaused = (Boolean)((PeerSettingChangedEvent)e).getNewSetting();
				refresh();
			}
		}
		public void peerInterruptableEvent(PeerInterruptableEvent e) { /* ignore this */ }
	}
	
	public void init(Peer thePeer) {
		if (thePeer != this.peer && this.peer != null)
			this.peer.unregisterPeerEventListener(this.el);
		this.peer = thePeer;
		try {
			if (!this.notWorking && this.peer != null) {
				this.peer.registerPeerEventListener(this.el);
				this.statePaused = this.peer.isCrawlingPaused();
				this.setEnabled(true);
				this.refresh();
			} else {
				this.setEnabled(false);
			}
		} catch (RemoteException e) {
			this.cfg.log(e);
			this.setEnabled(false);
			this.notWorking = true;
		}
	}
	
	private void refresh() {
		this.setIcon((this.statePaused) ? (start) : (pause));
		this.setToolTipText((this.statePaused) ? ("Crawlen fortsetzen") : ("Crawlen pausieren"));
		this.updateUI();
	}
	
	private void initialize() {
        this.setPreferredSize(new Dimension(24, 24));
        this.setMaximumSize(new Dimension(24, 24));
        this.setMinimumSize(new Dimension(24, 24));
        this.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		try {
        			peer.setCrawlStateAll(statePaused);
            		statePaused = !statePaused;
            		refresh();
        		} catch(RemoteException ee) { cfg.log(ee); }
        	}
    	});
        this.setBorderPainted(false);
        this.setEnabled(false);
		this.refresh();
	}
}
