/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package plugins;

import general.Config;

import java.awt.event.KeyEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.io.IOException;

import javax.swing.event.TreeExpansionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.SwingConstants;

import java.rmi.RemoteException;
import java.util.ArrayList;


import javax.swing.JToggleButton;

import views.PanelPlugin;
import yacy.Peer;
import yacy.datatypes.ShareElement;
import yacy.datatypes.ShareFolder;
import yacy.event.PeerEvent;
import yacy.event.PeerEventListener;
import yacy.event.PeerInterruptableEvent;
import yacy.event.PeerTransferFinishedEvent;

public class FileShare extends JPanel implements views.PanelPlugin {
	
	private static final long	serialVersionUID	= 1L;
	
	private JPanel 		jPanel2 		= null;
	private JSplitPane 	jSplitPane 		= null;
	private JPanel 		jPanel3 		= null;
	private JPanel 		jPanel4 		= null;
	private JLabel 		jLabel 			= null;
	private JLabel 		jLabel1 		= null;
	private JPanel 		jPanel5 		= null;
	private JButton 	jButton 		= null;
	private JButton 	jButton1 		= null;
	private JButton 	jButton2 		= null;
	private JPanel 		jPanel6 		= null;
	private JButton 	jButton3 		= null;
	private JButton 	jButton4 		= null;
	private JPanel 		jPanel 			= null;
	private JList 		dList 			= null;
	private JComboBox 	jComboBox 		= null;
	private JComboBox 	jComboBox1 		= null;
	private JScrollPane	jScrollPane1 	= null;
	private JScrollPane jScrollPane2 	= null;
	
	private static final String PluginName 			= "FileShare";
	private static final String PluginVersion 		= "0.1";
	private static final int 	PluginUpdateDelay 	= 0;	// no continous updates needed
	
	private Peer 	peer 	= null;
	private Config 	cfg 	= null;
	private JTree 	local 	= null;
	private JTree 	remote 	= null;
	
	private DefaultMutableTreeNode 	rpath 				= null;
	private DefaultMutableTreeNode 	lpath 				= null;
	private TreePath 				currentlyOpening 	= null;
	private JPanel 					jPanel1 			= null;
	private JLabel 					jLabel2 			= null;
	private JProgressBar 			jProgressBar 		= null;
	private JToggleButton 			jToggleButton 		= null;
	private Thread 					transferRunning 	= null;
	
	public final String getDescription() 		{ return PluginName; }
	public String 		getVersion() 			{ return PluginVersion; }
	public String 		getMenuPath() 			{ return PanelPlugin.ADMIN; }
	public int 			getUpdateDelay() 		{ return PluginUpdateDelay; }
	public int 			getNeededYacyRevision() { return 2857; }
	public ImageIcon 	getMenuIcon() 			{ return new ImageIcon(); }
	public String		getMenuToolTip() 		{ return ""; }
	public void 		doUpdate() 				{  }
	
	public void init(Peer peer, Config cfg) {
		this.peer = peer;
		this.peer.registerPeerEventListener(new PeerEventListener() {
			public void peerEvent(PeerEvent e) {
				if (e instanceof PeerTransferFinishedEvent) {
					transferRunning = null;
					refreshRemote();
				}
			}
			public void peerInterruptableEvent(PeerInterruptableEvent e) {
				if (transferRunning != null)
					e.interrupt("Transfer running", transferRunning);
			}
		});
		this.cfg = cfg;
		if (super.getWidth() > 0) {
			jSplitPane.setDividerLocation(super.getWidth() / 2);
			this.updateUI();
		}
		refreshLocal();
		refreshRemote();
	}
	
	private void refreshLocal() {
		refreshLocal("/");
	}
	
	private void refreshLocal(String dir) {
		lpath = getLocalDir(dir);
		local = null;
		local = getLocal();
		jScrollPane1.setViewportView(local);
		jSplitPane.updateUI();
	}
	
	private void refreshRemote() {
		refreshRemote("/");
	}
	
	private void refreshRemote(String dir) {
		rpath = getRemoteDir(dir);
		remote = null;
		remote = getRemote();
		jScrollPane2.setViewportView(remote);
		jSplitPane.updateUI();
	}
	
	private DefaultMutableTreeNode[] getRemoteDirContents(String path) {
		ArrayList<DefaultMutableTreeNode> r = new ArrayList<DefaultMutableTreeNode>();
		try {
			for (ShareElement d : peer.getShareDirList(path)) {
				DefaultMutableTreeNode f = new DefaultMutableTreeNode(d.getName());
				if (d instanceof ShareFolder)
					f.add(new DefaultMutableTreeNode("dummy"));		// to display it as a folder
				r.add(f);
			}
		} catch (RemoteException e) { cfg.log(e); return new DefaultMutableTreeNode[] {  }; }
		return r.toArray(new DefaultMutableTreeNode[r.size()]);
	}
	
	private DefaultMutableTreeNode getRemoteDir(String path) {
		DefaultMutableTreeNode r = new DefaultMutableTreeNode();
		try {
			for (ShareElement d : peer.getShareDirList(path)) {
				DefaultMutableTreeNode f = new DefaultMutableTreeNode(d.getName());
				if (d instanceof ShareFolder)
					f.add(new DefaultMutableTreeNode("dummy"));
				r.add(f);
			}
		} catch (RemoteException e) { cfg.log(e); }
		return r;
	}
	
	private DefaultMutableTreeNode[] getLocalDirContents(String path) {		
		File p = new File(path);
		ArrayList<DefaultMutableTreeNode> r = new ArrayList<DefaultMutableTreeNode>();
		if (!p.isDirectory() || !p.canRead())
			return new DefaultMutableTreeNode[] {  };
		
		boolean separate = path.length() > 1 && !path.endsWith(File.separator);
		for (String d : p.list()) {
			DefaultMutableTreeNode akt = new DefaultMutableTreeNode(d);
			if (new File(path + (separate ? File.separator : "") + d ).isDirectory())
				akt.add(new DefaultMutableTreeNode("dummy"));
			r.add(akt);
		}
		return r.toArray(new DefaultMutableTreeNode[r.size()]);
	}
	
	private DefaultMutableTreeNode getLocalDir(String path) {
		DefaultMutableTreeNode r = new DefaultMutableTreeNode(path);
		DefaultMutableTreeNode[] d = getLocalDirContents(path);
		for (int i=0; i<d.length; i++)
			r.add(d[i]);
		return r;
	}

	private String tree2String(TreePath d) {
		if (d == null) return "/";
		String r = "";
		for (int i=1; i<d.getPathCount(); i++)
			r += "/" + d.getPath()[i].toString();
		return r;
	}
	
	public FileShare() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setSize(551, 498);
		this.setLayout(new BorderLayout());
		this.add(getJPanel2(), BorderLayout.CENTER);
		this.add(getJPanel(), BorderLayout.SOUTH);
	}

	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
			gridBagConstraints41.gridx = 0;
			gridBagConstraints41.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints41.gridy = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints.gridx = 0;
			jPanel2 = new JPanel();
			jPanel2.setLayout(new GridBagLayout());
			jPanel2.add(getJSplitPane(), gridBagConstraints);
			jPanel2.add(getJPanel1(), gridBagConstraints41);
		}
		return jPanel2;
	}

	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setDividerLocation(255);
			jSplitPane.setDividerSize(8);
			jSplitPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
			jSplitPane.setLeftComponent(getJPanel4());
			jSplitPane.setRightComponent(getJPanel3());
		}
		return jSplitPane;
	}

	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridy = 4;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.gridwidth = 2;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints8.gridy = 3;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.gridwidth = 2;
			gridBagConstraints8.insets = new Insets(3, 5, 3, 3);
			gridBagConstraints8.gridx = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.fill = GridBagConstraints.VERTICAL;
			gridBagConstraints5.insets = new Insets(3, 2, 0, 5);
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			gridBagConstraints5.gridy = 2;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.anchor = GridBagConstraints.EAST;
			gridBagConstraints3.insets = new Insets(3, 5, 3, 5);
			gridBagConstraints3.gridy = 2;
			jLabel = new JLabel();
			jLabel.setText("Remote");
			jPanel3 = new JPanel();
			jPanel3.setLayout(new GridBagLayout());
			jPanel3.add(jLabel, gridBagConstraints3);
			jPanel3.add(getJPanel5(), gridBagConstraints5);
			jPanel3.add(getJComboBox1(), gridBagConstraints8);
			jPanel3.add(getJScrollPane2(), gridBagConstraints1);
		}
		return jPanel3;
	}

	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.fill = GridBagConstraints.BOTH;
			gridBagConstraints9.weighty = 1.0;
			gridBagConstraints9.gridy = 3;
			gridBagConstraints9.gridx = 1;
			gridBagConstraints9.gridwidth = 2;
			gridBagConstraints9.weightx = 1.0;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridy = 2;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.gridwidth = 2;
			gridBagConstraints7.insets = new Insets(3, 3, 3, 5);
			gridBagConstraints7.gridx = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 2;
			gridBagConstraints6.anchor = GridBagConstraints.EAST;
			gridBagConstraints6.insets = new Insets(3, 5, 0, 2);
			gridBagConstraints6.gridy = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.anchor = GridBagConstraints.WEST;
			gridBagConstraints4.insets = new Insets(3, 5, 3, 5);
			gridBagConstraints4.gridy = 0;
			jLabel1 = new JLabel();
			jLabel1.setText("Local");
			jPanel4 = new JPanel();
			jPanel4.setLayout(new GridBagLayout());
			jPanel4.add(jLabel1, gridBagConstraints4);
			jPanel4.add(getJPanel6(), gridBagConstraints6);
			jPanel4.add(getJComboBox(), gridBagConstraints7);
			jPanel4.add(getJScrollPane1(), gridBagConstraints9);
		}
		return jPanel4;
	}

	private JPanel getJPanel5() {
		if (jPanel5 == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setHgap(3);
			flowLayout.setAlignment(FlowLayout.LEFT);
			flowLayout.setVgap(0);
			jPanel5 = new JPanel();
			jPanel5.setMinimumSize(new Dimension(90, 24));
			jPanel5.setMaximumSize(new Dimension(32767, 24));
			jPanel5.setLayout(flowLayout);
			jPanel5.setPreferredSize(new Dimension(160, 24));
			jPanel5.add(getJButton1(), null);
			jPanel5.add(getJButton(), null);
			jPanel5.add(getJButton2(), null);
		}
		return jPanel5;
	}
	
	private boolean confirm(String question, String title) {
		return (JOptionPane.showConfirmDialog(
				this,
				question,
				title,
				JOptionPane.YES_NO_OPTION) == 0);
	}

	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setPreferredSize(new Dimension(24, 24));
			jButton.setToolTipText("Gewählte Datei(en) löschen");
			jButton.setIcon(new ImageIcon("ressources/fileclose.png"));
			jButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (remote.getSelectionCount() == 0) return;
					if (!confirm("Möchten Sie die Dateien wirklich löschen?", "Dateien löschen")) return;
					new Thread() {
						public void run() {
							for (TreePath selpath : remote.getSelectionPaths()) {
								cfg.log.addEntry("Deleting remote share: " + tree2String(selpath) + " ...", this.toString(), true);
								try {
									peer.deleteShareFile(tree2String(selpath));
								} catch (RemoteException ee) { cfg.log(ee); }
								refreshRemote();
								cfg.log.addEntry("Deletion of " + tree2String(selpath) + " finished", this.toString(), false);
							}
						}
					}.start();
				}
			});
		}
		return jButton;
	}

	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setIcon(new ImageIcon("ressources/2leftarrow.png"));
			jButton1.setMnemonic(KeyEvent.VK_UNDEFINED);
			jButton1.setToolTipText("Gewählte Datei(en) herunterladen");
			jButton1.setPreferredSize(new Dimension(24, 24));
			jButton1.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
				}
			});
		}
		return jButton1;
	}

	private JButton getJButton2() {
		if (jButton2 == null) {
			jButton2 = new JButton();
			jButton2.setPreferredSize(new Dimension(24, 24));
			jButton2.setToolTipText("Kommentar der Datei editieren");
			jButton2.setIcon(new ImageIcon("ressources/text_left.png"));
			jButton2.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (remote.getSelectionCount() != 1)
						return;
					
					String newComment = JOptionPane.showInputDialog("Neuer Kommentar");
					boolean indexFile = JOptionPane.showConfirmDialog(
							null,
							"Soll die Datei und der neue Kommentar in den Index aufgenommen werden?",
							"Datei indexieren?",
							JOptionPane.YES_NO_OPTION) == 0;
					
					try {
						peer.editShareFileComment(tree2String(remote.getSelectionPath()), newComment, indexFile);
					} catch (RemoteException ee) { cfg.log(ee); }
				}
			});
		}
		return jButton2;
	}

	private JPanel getJPanel6() {
		if (jPanel6 == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(FlowLayout.RIGHT);
			flowLayout1.setVgap(0);
			flowLayout1.setHgap(3);
			jPanel6 = new JPanel();
			jPanel6.setMinimumSize(new Dimension(109, 24));
			jPanel6.setMaximumSize(new Dimension(32767, 24));
			jPanel6.setLayout(flowLayout1);
			jPanel6.setPreferredSize(new Dimension(160, 24));
			jPanel6.add(getJToggleButton(), null);
			jPanel6.add(getJButton4(), null);
			jPanel6.add(getJButton3(), null);
		}
		return jPanel6;
	}

	private JButton getJButton3() {
		if (jButton3 == null) {
			jButton3 = new JButton();
			jButton3.setPreferredSize(new Dimension(24, 24));
			jButton3.setToolTipText("Gewählte Datei(en) zu Peer übertragen");
			jButton3.setIcon(new ImageIcon("ressources/2rightarrow.png"));
			jButton3.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (local.getSelectionCount() == 0) return;
					File d = null;
					String path = null;
					
					for (TreePath selpath : local.getSelectionPaths()) {
						d = new File(tree2String(selpath));
						path = "/";
						if (remote.getSelectionCount() > 0 &&
								!((DefaultMutableTreeNode)remote.getSelectionPath().getLastPathComponent()).isLeaf())
							path = tree2String(remote.getSelectionPath());
						
						try {
							transferRunning = peer.uploadShareFile(d, path, "", jToggleButton.isSelected());
						} catch (IOException e1) { cfg.log(e1); }	
					}
				}
			});
		}
		return jButton3;
	}

	private JButton getJButton4() {
		if (jButton4 == null) {
			jButton4 = new JButton();
			jButton4.setPreferredSize(new Dimension(24, 24));
			jButton4.setToolTipText("Gewählte Datei(en) löschen");
			jButton4.setIcon(new ImageIcon("ressources/fileclose.png"));
			jButton4.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (!confirm("Möchten Sie die Dateien wirklich löschen?", "Dateien löschen"))
						return;
					if (local.getSelectionCount() == 0)
						return;
					for (TreePath selpath : local.getSelectionPaths()) {
						new File(tree2String(selpath)).delete();
						refreshLocal();
					}
				}
			});
		}
		return jButton4;
	}
	
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getDList(), java.awt.BorderLayout.CENTER);
		}
		return jPanel;
	}

	private JList getDList() {
		if (dList == null) {
			dList = new JList();
			dList.setPreferredSize(new Dimension(0, 80));
			dList.setVisible(false);
		}
		return dList;
	}

	/**
	 * This method initializes jComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBox() {
		if (jComboBox == null) {
			jComboBox = new JComboBox();
			jComboBox.setEditable(true);
			jComboBox.setMinimumSize(new Dimension(32, 19));
			jComboBox.setMaximumSize(new Dimension(32767, 19));
			jComboBox.setPreferredSize(new Dimension(32, 19));
			jComboBox.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					if (e.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER) {
						// TODO: go to selected position in local file-system
					}
				}
			});
		}
		return jComboBox;
	}

	/**
	 * This method initializes jComboBox1	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBox1() {
		if (jComboBox1 == null) {
			jComboBox1 = new JComboBox();
			jComboBox1.setEditable(true);
			jComboBox1.setMinimumSize(new Dimension(32, 19));
			jComboBox1.setMaximumSize(new Dimension(32767, 19));
			jComboBox1.setPreferredSize(new Dimension(32, 19));
			jComboBox1.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					if (e.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER) {
						// TODO: go to selected position in remote file-system
					}
				}
			});
		}
		return jComboBox1;
	}
	
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getLocal());
		}
		return jScrollPane1;
	}
	
	private JTree getLocal() {
		JTree r = new JTree(lpath);
		r.setRootVisible(false);
		r.setShowsRootHandles(true);
		r.addTreeExpansionListener(new javax.swing.event.TreeExpansionListener() {
			public void treeExpanded(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[where.getPath().length-1].toString() == null)
					currentlyOpening = null;
				jProgressBar.setVisible(false);
				return;
			}
			public void treeCollapsed(TreeExpansionEvent e) {  }
		});
		r.addTreeWillExpandListener(new javax.swing.event.TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[where.getPath().length-1].toString() == null)
					return;
				else
					currentlyOpening = e.getPath();
				
				jProgressBar.setVisible(true);
				jProgressBar.updateUI();
				jPanel1.updateUI();
				((DefaultMutableTreeNode)where.getLastPathComponent()).removeAllChildren();
				DefaultMutableTreeNode[] listing = getLocalDirContents(tree2String(where));
				for (int h=0; h<listing.length; h++)
					((DefaultMutableTreeNode)where.getLastPathComponent()).add(listing[h]);
			}
			public void treeWillCollapse(TreeExpansionEvent e) {  }
		});
		return r;
	}
	
	private JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new JScrollPane();
			jScrollPane2.setViewportView(getRemote());
		}
		return jScrollPane2;
	}
	
	/*
	private class RNode extends DefaultMutableTreeNode {
		
		private boolean isUpdated = false;
		private boolean isDirectory = false;
		private String nodeName = null;
		
		public RNode(Object name) { this.nodeName = name.toString(); }
		
		public void doUpdate(DefaultMutableTreeNode[] contents) {
			if (!this.isUpdated && this.getChildCount() > 0 && !this.isDirectory)
				for (int i=0; i<contents.length; this.add(contents[i++])) ;
			this.isUpdated = true;
		}
		
		public boolean isDirectory() { return this.isDirectory; }
		
		public boolean isUpdated() { return this.isUpdated; }
		
		public void setDirectory(boolean yes) {
			if (yes)
				this.add(new DefaultMutableTreeNode("dummy"));
			this.isDirectory = yes;
		}
		
		public String toString() { return this.nodeName; }
	}
	*/
	
	private JTree getRemote() {
		JTree r = new JTree(rpath);
		r.setRootVisible(false);
		r.setShowsRootHandles(true);
		r.addTreeExpansionListener(new javax.swing.event.TreeExpansionListener() {
			public void treeExpanded(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[1].toString() == null)
					currentlyOpening = null;
				jProgressBar.setVisible(false);
				return;
			}
			public void treeCollapsed(TreeExpansionEvent e) {  }
		});
		r.addTreeWillExpandListener(new javax.swing.event.TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[1].toString() == null)
					return;
				else
					currentlyOpening = e.getPath();
				
				jProgressBar.setVisible(true);
				jPanel1.updateUI();
				((DefaultMutableTreeNode)where.getLastPathComponent()).removeAllChildren();
				DefaultMutableTreeNode[] listing = getRemoteDirContents(tree2String(where));
				for (int h=0; h<listing.length; h++)
					((DefaultMutableTreeNode)where.getLastPathComponent()).add(listing[h]);
			}
			public void treeWillCollapse(TreeExpansionEvent e) {  }
		});
		return r;
	}
	
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jLabel2 = new JLabel();
			jLabel2.setPreferredSize(new Dimension(38, 21));
			jLabel2.setText("JLabel");
			jLabel2.setHorizontalAlignment(SwingConstants.LEFT);
			jPanel1 = new JPanel();
			jPanel1.setLayout(new BorderLayout());
			jPanel1.add(jLabel2, BorderLayout.CENTER);
			jPanel1.add(getJProgressBar(), BorderLayout.EAST);
		}
		return jPanel1;
	}
	
	private JProgressBar getJProgressBar() {
		if (jProgressBar == null) {
			jProgressBar = new JProgressBar();
			jProgressBar.setString("");
			jProgressBar.setPreferredSize(new Dimension(60, 14));
			jProgressBar.setVisible(false);
			jProgressBar.setIndeterminate(true);
		}
		return jProgressBar;
	}
	
	private JToggleButton getJToggleButton() {
		if (jToggleButton == null) {
			jToggleButton = new JToggleButton();
			jToggleButton.setPreferredSize(new Dimension(24, 24));
			jToggleButton.setMaximumSize(new Dimension(24, 24));
			jToggleButton.setToolTipText("Datei(en) beim Hochladen indexieren");
			jToggleButton.setMinimumSize(new Dimension(24, 24));
		}
		return jToggleButton;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
