/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package plugins;

import general.Config;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import tools.DGramm;
import views.PanelPlugin;

import yacy.Peer;


public class Status extends JPanel implements views.PanelPlugin {
	
	private static final long serialVersionUID = 1L;
	
	private JPanel 			jPanel 		= null;
	private JLabel 			jLabel 		= null;
	private JLabel 			links 		= null;
	private JLabel 			jLabel2 	= null;
	private JLabel 			words 		= null;
	private JLabel 			jLabel4 	= null;
	private JLabel 			jLabel5 	= null;
	private JProgressBar 	memory 		= null;
	private JLabel 			jLabel6 	= null;
	private JLabel 			pStatus 	= null;
	private JLabel 			ppm 		= null;
	private JLabel 			jLabel11 	= null;
	private JProgressBar 	ramCache 	= null;
	private JLabel 			title 		= null;
	private JPanel 			jPanel2 	= null;
	private JLabel 			jLabel15 	= null;
	private JLabel 			peerCount 	= null;
	private JLabel 			jLabel17 	= null;
	private JLabel 			linkCount 	= null;
	private JLabel 			jLabel19 	= null;
	private JLabel 			wordCount 	= null;
	private JLabel 			jLabel21 	= null;
	private DGramm 			jPanel4 	= null;
	private JLabel 			aktMem 		= null;
	private JLabel 			aktRamCache = null;
	private JLabel 			jLabel1 	= null;
	private JLabel 			peerStatus 	= null;
	
	private static final String 					PluginName = "Peer Status"; 
	private static final String 					PluginVersion = "0.1"; 
	private static final HashMap<String,String[]> 	pTypes = new HashMap<String,String[]>();
	private static final ArrayList<String> 			pStati = new ArrayList<String>();
	private static final ArrayList<Color>			pStatiColors = new ArrayList<Color>();
	static {
		pTypes.put(Peer.STATE_VIRGIN, new String[] { "Virgin",
				"Der Peer konnte noch keine Verbindung mit dem YaCy-Netzwerk herstellen." });
		pTypes.put(Peer.STATE_JUNIOR, new String[] { "Junior",
				"Der Peer ist mit dem YaCy-Netzwerk verbunden, kann von außen allerdings\n" +
				"nicht erreicht werden. Dies kann zum Beispiel Firewall-Einstellungen oder\n" +
				"Remote-Proxies liegen." });
		pTypes.put(Peer.STATE_SENIOR, new String[] { "Senior",
				"Der Peer ist mit dem YaCy-Netzwerk verbunden und trägt zum globalen Index bei." });
		pTypes.put(Peer.STATE_PRINCIPAL, new String[] { "Prinicipal",
				"Der Peer ist mit dem YaCy-Netzwerk verbunden und trägt zum globalen Index bei.\n" + 
				"Außerdem hilft er anderen Peers dabei, das Netz zu finden." });
		pStati.add("Online");
		pStati.add("No Internet available");
		pStati.add("Not online");
		pStatiColors.add(Color.GREEN);
		pStatiColors.add(Color.ORANGE);
		pStatiColors.add(Color.RED);
	}

	private int 	maxMemory = 0;
	private int 	maxRamCache = 0;
	private Peer 	peer = null;
	private Config 	cfg = null;
	
	public Status() {
		super();
		initialize();
		this.repaint();
	}
	
	private void initialize() {
		BorderLayout borderLayout = new BorderLayout();
		borderLayout.setVgap(5);
		borderLayout.setHgap(0);
		this.setLayout(borderLayout);
		this.setSize(501, 409);
		this.add(getJPanel(), BorderLayout.CENTER);
		this.add(getJPanel2(), BorderLayout.SOUTH);
	}
	
	public ImageIcon 	getMenuIcon() 			{ return new ImageIcon(); }
	public String		getMenuToolTip() 		{ return null; }
	public String 		getVersion() 			{ return PluginVersion; }
	public String 		getDescription() 		{ return PluginName; }
	public String 		getMenuPath() 			{ return PanelPlugin.STATUS; }
	public int			getUpdateDelay() 		{ return 1000; }
	public int 			getNeededYacyRevision() { return 2920; }
	
	public void init(Peer peer, Config cfg) {
		this.peer = peer;
		this.cfg = cfg;
		title.setText(peer.getName());
		try {
			maxRamCache = peer.getRamCacheSize();
			maxMemory = peer.getMemoryMaxMB();
		} catch (RemoteException e) { cfg.log(e); }
	}
	
	public void doUpdate() {
		cfg.logDebug(" Updating Status.");
		
		long tLinks = 0;
		long tWords = 0;
		int tPPM = 0;
		int tpStatus = -1;
		String tType = null;
		int tpCountActive = 0;
		long tLinksActive = 0;
		long tWordsActive = 0;
		long memTotal = 0;
		long memFree = 0;
		long memMax = 0;
		int tRamCacheNow = 0;
		int tRamCacheSize = 0;
		
		tLinks = peer.getLinks();
		tWords = peer.getWords();
		tPPM = peer.getPPM();
		tpStatus = peer.getPeerStatus();
		tType = peer.getType();
		tpCountActive = peer.getPeerCount(Peer.PEERS_ACTIVE);
		tLinksActive = peer.getLinks(Peer.PEERS_ACTIVE);
		tWordsActive = peer.getWords(Peer.PEERS_ACTIVE);
		memMax = peer.getMemoryMax();
		try {
			tRamCacheSize = peer.getRamCacheSize();
			tRamCacheNow = peer.getRamCacheNow();
			memFree = peer.getMemoryFree();
			memTotal = peer.getMemoryTotal();
		} catch (RemoteException e) { cfg.log(e); }
		
		links.setText(LongToSeparatedString(tLinks));
		words.setText(LongToSeparatedString(tWords));
		jPanel4.addValue(tPPM, 0);
		jPanel4.repaint();
		ppm.setText(Integer.toString(tPPM));
		pStatus.setText(pStati.get(tpStatus));
		pStatus.setForeground(pStatiColors.get(tpStatus));
		peerStatus.setText(pTypes.get(tType)[0]);
		peerStatus.setToolTipText(pTypes.get(tType)[1]);
		peerCount.setText(Integer.toString(tpCountActive));
		linkCount.setText(LongToSeparatedString(tLinksActive));
		wordCount.setText(LongToSeparatedString(tWordsActive));
		memory.setValue(Math.round(((float)(memTotal - memFree) / (float)memMax) * 100));
		memory.setToolTipText("Freier Speicher: " + ((memMax - (memTotal - memFree)) / (1024 * 1024)) + " MB");
		aktMem.setText(Long.toString((memTotal - memFree) / (1024  * 1024)) + " / " + Integer.toString(maxMemory) + " MB");
		ramCache.setValue(Math.round(((float)tRamCacheNow / (float)tRamCacheSize * 100)));
		aktRamCache.setText(LongToSeparatedString(tRamCacheNow) + " / " + LongToSeparatedString(maxRamCache) + " Words");
		
		jPanel.updateUI();
	}
	
	private String LongToSeparatedString(long l) {
		String oldnumber = Long.toString(l);
		if(oldnumber.length() < 4) return oldnumber;
		
		String s = "";		
		for(int i=oldnumber.length()-3;i>-3;i=i-3)
			s = oldnumber.substring((i>0)?i:0,i+3)+(s.length()>0?".":"")+s;
		
		return s;
	}

	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints23 = new GridBagConstraints();
			gridBagConstraints23.gridx = 1;
			gridBagConstraints23.anchor = GridBagConstraints.WEST;
			gridBagConstraints23.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints23.gridy = 6;
			peerStatus = new JLabel();
			peerStatus.setText("Status");
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.gridx = 0;
			gridBagConstraints17.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints17.anchor = GridBagConstraints.EAST;
			gridBagConstraints17.gridy = 6;
			jLabel1 = new JLabel();
			jLabel1.setText("Peer Status:");
			GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
			gridBagConstraints22.gridx = 2;
			gridBagConstraints22.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints22.gridy = 5;
			aktRamCache = new JLabel();
			aktRamCache.setText("akt");
			GridBagConstraints gridBagConstraints110 = new GridBagConstraints();
			gridBagConstraints110.gridx = 2;
			gridBagConstraints110.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints110.gridy = 4;
			aktMem = new JLabel();
			aktMem.setText("akt");
			GridBagConstraints gridBagConstraints19 = new GridBagConstraints();
			gridBagConstraints19.gridx = 1;
			gridBagConstraints19.fill = GridBagConstraints.BOTH;
			gridBagConstraints19.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints19.gridy = 3;
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints13.gridwidth = 3;
			gridBagConstraints13.gridy = 0;
			title = new JLabel();
			title.setText("peerName");
			title.setFont(new Font("Dialog", Font.BOLD, 18));
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
			gridBagConstraints15.gridx = 1;
			gridBagConstraints15.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints15.gridy = 5;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.gridx = 0;
			gridBagConstraints14.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints14.anchor = GridBagConstraints.EAST;
			gridBagConstraints14.gridy = 5;
			jLabel11 = new JLabel();
			jLabel11.setText("RAM Cache:");
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 2;
			gridBagConstraints11.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints11.anchor = GridBagConstraints.WEST;
			gridBagConstraints11.gridy = 3;
			ppm = new JLabel();
			ppm.setText("ppm");
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 1;
			gridBagConstraints9.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints9.anchor = GridBagConstraints.WEST;
			gridBagConstraints9.weighty = 0.0D;
			gridBagConstraints9.gridy = 7;
			pStatus = new JLabel();
			pStatus.setText("Icon");
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints8.anchor = GridBagConstraints.EAST;
			gridBagConstraints8.gridy = 7;
			jLabel6 = new JLabel();
			jLabel6.setText("Verbindung:");
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 1;
			gridBagConstraints7.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints7.gridy = 4;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints6.anchor = GridBagConstraints.EAST;
			gridBagConstraints6.gridy = 4;
			jLabel5 = new JLabel();
			jLabel5.setText("Speicher:");
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints4.anchor = GridBagConstraints.EAST;
			gridBagConstraints4.gridy = 3;
			jLabel4 = new JLabel();
			jLabel4.setText("PPM:");
			jLabel4.setToolTipText("Pages per Minute");
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints3.anchor = GridBagConstraints.WEST;
			gridBagConstraints3.gridy = 2;
			words = new JLabel();
			words.setText("JLabel");
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints2.anchor = GridBagConstraints.EAST;
			gridBagConstraints2.gridy = 2;
			jLabel2 = new JLabel();
			jLabel2.setText("Words:");
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints1.anchor = GridBagConstraints.WEST;
			gridBagConstraints1.gridy = 1;
			links = new JLabel();
			links.setText("JLabel");
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.gridy = 1;
			jLabel = new JLabel();
			jLabel.setText("Links:");
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			jPanel.setPreferredSize(new Dimension(0, 230));
			jPanel.add(jLabel, gridBagConstraints);
			jPanel.add(links, gridBagConstraints1);
			jPanel.add(jLabel2, gridBagConstraints2);
			jPanel.add(words, gridBagConstraints3);
			jPanel.add(jLabel4, gridBagConstraints4);
			jPanel.add(jLabel5, gridBagConstraints6);
			jPanel.add(getMemory(), gridBagConstraints7);
			jPanel.add(jLabel6, gridBagConstraints8);
			jPanel.add(pStatus, gridBagConstraints9);
			jPanel.add(ppm, gridBagConstraints11);
			jPanel.add(jLabel11, gridBagConstraints14);
			jPanel.add(getRamCache(), gridBagConstraints15);
			jPanel.add(title, gridBagConstraints13);
			jPanel.add(getJPanel4(), gridBagConstraints19);
			jPanel.add(aktMem, gridBagConstraints110);
			jPanel.add(aktRamCache, gridBagConstraints22);
			jPanel.add(jLabel1, gridBagConstraints17);
			jPanel.add(peerStatus, gridBagConstraints23);
		}
		return jPanel;
	}

	private JProgressBar getMemory() {
		if (memory == null) {
			memory = new JProgressBar();
			memory.setToolTipText("Maximal: ");
			memory.setMinimum(0);
			memory.setMaximum(100);
		}
		return memory;
	}
	
	private JProgressBar getRamCache() {
		if (ramCache == null) {
			ramCache = new JProgressBar();
		}
		return ramCache;
	}

	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jLabel21 = new JLabel();
			jLabel21.setText("Words");
			wordCount = new JLabel();
			wordCount.setText("JLabel");
			jLabel19 = new JLabel();
			jLabel19.setText("Links");
			linkCount = new JLabel();
			linkCount.setText("JLabel");
			jLabel17 = new JLabel();
			jLabel17.setText("Peers");
			peerCount = new JLabel();
			peerCount.setText("JLabel");
			jLabel15 = new JLabel();
			jLabel15.setText("Aktiv:");
			jPanel2 = new JPanel();
			jPanel2.setLayout(new FlowLayout());
			jPanel2.setPreferredSize(new Dimension(0, 25));
			jPanel2.add(jLabel15, null);
			jPanel2.add(peerCount, null);
			jPanel2.add(jLabel17, null);
			jPanel2.add(linkCount, null);
			jPanel2.add(jLabel19, null);
			jPanel2.add(wordCount, null);
			jPanel2.add(jLabel21, null);
		}
		return jPanel2;
	}
	
	private DGramm getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new DGramm(1,50);
			jPanel4.setXFactor(1,0);
			jPanel4.setGraphFilled(true, 0);
			jPanel4.setPreferredSize(new Dimension(0,50));
			jPanel4.setToolTipText("PPM gibt die Anzahl der indexierten Seiten pro Minute an.");
			jPanel4.addValue(15, 0);// } catch (Exception e) {  }
			jPanel4.addValue(0, 0);// } catch (Exception e) {  }
		}
		return jPanel4;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
