/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package plugins;

import general.Config;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;


import views.TrayPlugin;
import yacy.Peer;
import yacy.event.PeerEvent;
import yacy.event.PeerEventListener;
import yacy.event.PeerInterruptableEvent;
import yacy.event.PeerSettingChangedEvent;

public class TCrawlPauseResume extends JMenuItem implements TrayPlugin {
	
	private static final long serialVersionUID = 1L;
	
	private boolean notWorking 	= false;
	private boolean statePaused = false;
	private EL 		el 			= null;
	private Peer 	peer 		= null;
	private Config 	cfg 		= null;
	
	private static final ImageIcon pause = new ImageIcon("ressources/pause_small.png");
	private static final ImageIcon start = new ImageIcon("ressources/start_small.png");
	
	
	public int 		getNeededYacyRevision() { return 2958; }
	public String 	getVersion() 			{ return "0.1"; }
	
	private class EL implements PeerEventListener {
		public void peerEvent(PeerEvent e) {
			if (e instanceof PeerSettingChangedEvent &&
					((PeerSettingChangedEvent)e).getDescription().indexOf("queue state") > -1) {
				statePaused = (Boolean)((PeerSettingChangedEvent)e).getNewSetting();
				refresh();
			}
		}
		public void peerInterruptableEvent(PeerInterruptableEvent e) { /* ignore this */ }
	}
	
	public void init(Peer thePeer, Config cfg) {
		this.peer = thePeer;
		this.cfg = cfg;
		this.peer.registerPeerEventListener(this.el);
		try {
			if (!this.notWorking && this.peer != null) {
				this.statePaused = this.peer.isCrawlingPaused();
				this.setEnabled(true);
				this.refresh();
			} else {
				this.setEnabled(false);
			}
		} catch (RemoteException e) {
			cfg.log(e);
			this.setEnabled(false);
			this.notWorking = true;
		}
	}
	
	private void refresh() {
		this.setIcon((this.statePaused) ? (start) : (pause));
		this.setText((this.statePaused) ? ("Crawlen fortsetzen") : ("Crawlen pausieren"));
		this.updateUI();
	}
	
	public TCrawlPauseResume() { initialize(); }
	public TCrawlPauseResume(Icon icon) { super(icon); initialize(); }
	public TCrawlPauseResume(String text) { super(text); initialize(); }
	public TCrawlPauseResume(Action a) { super(a); initialize(); }
	public TCrawlPauseResume(String text, Icon icon) { super(text, icon); initialize(); }
	public TCrawlPauseResume(String text, int mnemonic) { super(text, mnemonic); initialize(); }
	
	private void initialize() {
		this.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		try {
        			peer.setCrawlStateAll(statePaused);
            		statePaused = !statePaused;
            		refresh();
        		} catch(RemoteException ee) { cfg.log(ee); }
        	}
    	});
        this.setEnabled(false);
        this.el = new EL();
	}
}
