/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package views;

import general.Config;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import yacy.Codings;

public class ConnectDialog extends JDialog {
	
	private static final long serialVersionUID = 1L;
	
	private static final Insets STD_INSETS 		= new Insets(5, 5, 5, 5);
	private static final String DEFAULT_USER 	= "admin";
	private static final String DUMMY_PWD 		= "********";
	
	private JButton 		btnCancel 	= null;
	private JButton 		btnConnect 	= null;
	private JTextField 		textUser 	= null;
	private JPasswordField 	textPwd 	= null;
	private JTextField 		textPort 	= null;
	private JTextField 		textHost 	= null;
	private JComboBox 		cBoxSaved 	= null;
	
	private static final int width 	= 350;
	private static final int height = 205;
	private final Config cfg;
	
	private String 	user 	= null;
	private String 	pwd 	= null;
	private String 	host 	= null;
	private int 	port 	= 0;
	private boolean success = false;
	private boolean pwdChanged = false;
	
	public String 	getHost() 		{ return this.host; }
	public int 		getPort() 		{ return this.port; }
	public String 	getPwd() 		{ return this.pwd; }
	public boolean 	getSuccess() 	{ return this.success; }
	public String 	getUser() 		{ return this.user; }
	
	public ConnectDialog(Frame parent, Config cfg) {
		super(parent, true);
		this.cfg = cfg;
		initialize();
	}
	
	public boolean showDialog() {
		this.setVisible(true);
		return this.success;
	}
	
	private boolean check() {
		boolean failed = (this.getTextHost().getText().trim().length() == 0 ||
				this.getTextPwd().getPassword().length == 0 ||
				!this.getTextPort().getText().matches("\\d+") ||
				this.getTextUser().getText().trim().length() == 0);
		this.getBtnConnect().setEnabled(!failed);
		return !failed;
	}
	
	private void initialize() {
		this.setSize(width, height);
		this.setResizable(false);
		
		// place dialog in the middle of the screen
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation((int)(dim.getWidth() - width) / 2, (int)(dim.getHeight() - height) / 2);
		
		this.setLayout(new BorderLayout());
		this.setContentPane(getPanelEnter());
		check();
	}
	
	private JPanel getPanelSubmit() {
		JPanel jp = new JPanel();
		jp = new JPanel();
		jp.setLayout(new BorderLayout());
		jp.add(getBtnCancel(), BorderLayout.WEST);
		jp.add(getBtnConnect(), BorderLayout.EAST);
		return jp;
	}
	
	private class DChangedKeyAdapter extends KeyAdapter {
		public void keyReleased(KeyEvent e) {
			if (!ConnectDialog.this.pwdChanged) {
				ConnectDialog.this.textPwd.setText(null);
				ConnectDialog.this.textPwd.updateUI();
			}
			ConnectDialog.this.check();
		}
	}
	
	private JButton getBtnCancel() {
		if (btnCancel == null) {
			btnCancel = new JButton();
			btnCancel.setText("Abbrechen");
			this.btnCancel.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					ConnectDialog.this.success = false;
					ConnectDialog.this.setVisible(false);
				}
			});
		}
		return btnCancel;
	}
	
	private JButton getBtnConnect() {
		if (this.btnConnect == null) {
			this.btnConnect = new JButton();
			this.btnConnect.setText("Verbinden...");
			this.btnConnect.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					ConnectDialog.this.user = ConnectDialog.this.getTextUser().getText();
					ConnectDialog.this.host = ConnectDialog.this.getTextHost().getText();
					ConnectDialog.this.port = Integer.parseInt(ConnectDialog.this.getTextPort().getText());
					if (ConnectDialog.this.pwdChanged) {
						ConnectDialog.this.pwd = Codings.encodeMD5Hex(Codings.encodeString(
								ConnectDialog.this.user + ':' +
								new String(ConnectDialog.this.getTextPwd().getPassword())));
					} else {
						ConnectDialog.this.pwd = ConnectDialog.this.cfg.getConnect(
								ConnectDialog.this.host,
								ConnectDialog.this.port,
								ConnectDialog.this.user);
					}
					
					ConnectDialog.this.cfg.putConnect(
							ConnectDialog.this.host,
							ConnectDialog.this.port,
							ConnectDialog.this.user,
							ConnectDialog.this.pwd);
					
					ConnectDialog.this.success = true;
					ConnectDialog.this.setVisible(false);
				}
			});
			this.getRootPane().setDefaultButton(this.btnConnect);
		}
		return this.btnConnect;
	}
	
	private JPanel getPanelEnter() {
		JPanel jp = new JPanel();
		
		jp = new JPanel();
		jp.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();

		// Delete entry - button
		gbc.gridx = 2;
		gbc.gridy = 0;
		gbc.insets = STD_INSETS;
		JButton jb = new JButton(new ImageIcon("ressources/stop.png"));
		jb.setToolTipText("Eintrag entfernen");
		Dimension jbd = new Dimension(20, 20);
		jb.setPreferredSize(jbd);
		jb.setMinimumSize(jbd);
		jb.setMaximumSize(jbd);
		jb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ConnectItem ci = (ConnectItem)ConnectDialog.this.getCBoxSaved().getSelectedItem();
				if (ci != null)
					ConnectDialog.this.cfg.connects.remove(ci.toString());
				ConnectDialog.this.getCBoxSaved().removeItem(ci);
			}
		});
		jp.add(jb, gbc);
		
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx = 0;
		addJComponents(
				new JComponent[] {
						new JLabel("Gespeichert:"),
						new JLabel("Host:"),
						new JLabel("Port:"),
						new JLabel("Username:"),
						new JLabel("Passwort:")
				},
				gbc, true, jp);
		
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		gbc.gridx = 1;
		gbc.gridy = 0;
		jp.add(getCBoxSaved(), gbc);
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		addJComponents(
				new JComponent[] {
						getTextHost(),
						getTextPort(),
						getTextUser(),
						getTextPwd()
				},
				gbc, true, jp);
		
		gbc.gridx = 0;
		gbc.gridy = 5;
		gbc.gridwidth = 3;
		jp.add(getPanelSubmit(), gbc);
		
		return jp;
	}

	private static void addJComponents(JComponent[] jc,
			GridBagConstraints gbc, boolean vertical, JPanel to) {
		
		if (jc == null || gbc == null || to == null) return;
		for (int i=0; i<jc.length; i++) {
			to.add(jc[i], gbc);
			if (vertical) gbc.gridy++; else gbc.gridx++;
		}
	}
	
	private JTextField getTextUser() {
		if (this.textUser == null) {
			this.textUser = new JTextField();
			this.textUser.setMaximumSize(new Dimension(2147483647, 19));
			this.textUser.setText(DEFAULT_USER);
			this.textUser.addKeyListener(new DChangedKeyAdapter());
		}
		return this.textUser;
	}
	
	private JPasswordField getTextPwd() {
		if (this.textPwd == null) {
			this.textPwd = new JPasswordField();
			this.textPwd.setMaximumSize(new Dimension(2147483647, 19));
			this.textPwd.addKeyListener(new KeyAdapter() {
				public void keyReleased(KeyEvent e) {
					ConnectDialog.this.pwdChanged = true;
					check();
				}
			});
		}
		return this.textPwd;
	}
	
	private JTextField getTextPort() {
		if (this.textPort == null) {
			this.textPort = new JTextField();
			this.textPort.setMaximumSize(new Dimension(2147483647, 19));
			this.textPort.addKeyListener(new DChangedKeyAdapter());
		}
		return this.textPort;
	}
	
	private JTextField getTextHost() {
		if (this.textHost == null) {
			this.textHost = new JTextField();
			this.textHost.setMaximumSize(new Dimension(2147483647, 19));
			this.textHost.addKeyListener(new DChangedKeyAdapter());
		}
		return this.textHost;
	}
	
	private JComboBox getCBoxSaved() {
		if (this.cBoxSaved == null) {
			this.cBoxSaved = new JComboBox();
			this.cBoxSaved.setPreferredSize(new Dimension(32, 19));
			this.cBoxSaved.setMaximumSize(new Dimension(32767, 19));
			this.cBoxSaved.setMinimumSize(new Dimension(32, 19));
			this.cBoxSaved.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					ConnectItem ci = (ConnectItem)ConnectDialog.this.cBoxSaved.getSelectedItem();
					if (ci != null) {
						ConnectDialog.this.getTextHost().setText(ci.getHost());
						ConnectDialog.this.getTextPort().setText(ci.getPort());
						ConnectDialog.this.getTextUser().setText(ci.getUser());
						ConnectDialog.this.getTextPwd().setText(ci.getPwd());
					} else {
						ConnectDialog.this.getTextHost().setText("");
						ConnectDialog.this.getTextPort().setText("");
						ConnectDialog.this.getTextUser().setText(ConnectDialog.DEFAULT_USER);
						ConnectDialog.this.getTextPwd().setText("");
					}
					ConnectDialog.this.check();
				}
			});
		} else {
			this.cBoxSaved.removeAllItems();
		}
		for (String d : this.cfg.connects.keySet())
			this.cBoxSaved.addItem(new ConnectItem(d, this.cfg.connects.get(d)));
		
		return this.cBoxSaved;
	}
	
	private static class ConnectItem {
		private final String host;
		private final String port;
		private final String user;
		private final String pwd;
		
		public ConnectItem(String key, String pwd) {
			this.user = key.substring(0, key.indexOf('@'));
			this.host = key.substring(key.indexOf('@') + 1, key.indexOf(':'));
			this.port = key.substring(key.indexOf(':') + 1);
			this.pwd = (pwd.length() > 0) ? DUMMY_PWD : "";
		}
		
		public String getHost() { return this.host; }
		public String getPort() { return this.port; }
		public String getUser() { return this.user; }
		public String getPwd() { return this.pwd; }
		
		public String toString() {
			return this.user + '@' + this.host + ':' + this.port;
		}
	}
	
}
