/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package views;

import java.awt.GridBagLayout;
import javax.swing.JPanel;
import java.awt.Dimension;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.SwingConstants;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.JToggleButton;
import javax.swing.ButtonGroup;

import general.Config;
import javax.swing.JTextField;

/**
 * This class represents the view for customizing the YacyAdmin settings.
 */
public class Settings extends JPanel {
	
	private static final long serialVersionUID = 1L;
	
	private static final ImageIcon denied = new ImageIcon("ressources/remove.png");  //  @jve:decl-index=0:
	private static final Insets STD_INSETS = new Insets(5, 5, 5, 5);
	
	private ButtonGroup 	logShow 			= new ButtonGroup();
	private JCheckBox 		jCHttp11 			= null;
	private JCheckBox 		jCChunkedTransfers 	= null;
	private JCheckBox 		jCGzip 				= null;
	private JCheckBox 		jCBUpdateInterval 	= null;
	private JCheckBox 		jCProxy 			= null;
	private JComboBox 		jCAutoConnect 		= null;
	private JComboBox 		jCProtcol 			= null;
	private JLabel 			jLUpdInterval 		= null;
	private JLabel 			jLHideLogDelay 		= null;
	private JLabel 			jLConnectionTimeout = null;
	private JLabel 			jLPortStatus 		= null;
	private JRadioButton 	jRLogShowAlway 		= null;
	private JRadioButton 	jRLogHide 			= null;
	private JSlider 		jSLogHideDelay 		= null;
	private JSlider 		jSUpdateInterval 	= null;
	private JSlider 		jSConnectionTimeout = null;
	private JTextField 		jTHost 				= null;
	private JTextField 		jTPort 				= null;

	private Config 			cfg 				= null;

	public Settings(Config cfg) {
		super();
		this.cfg = cfg;
		initialize();
	}
	
	private void initialize() {
		
		// X: 1
		GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
		gridBagConstraints6.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints6.gridx = 1;
		gridBagConstraints6.gridy = 1;
		gridBagConstraints6.weightx = 1.0;
		gridBagConstraints6.gridwidth = 4;
		gridBagConstraints6.insets = STD_INSETS;
		GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
		gridBagConstraints14.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints14.gridy = 9;
		gridBagConstraints14.weightx = 1.0;
		gridBagConstraints14.gridwidth = 3;
		gridBagConstraints14.insets = STD_INSETS;
		gridBagConstraints14.gridx = 1;
		
		// X: 2
		GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
		gridBagConstraints41.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints41.gridx = 2;
		gridBagConstraints41.gridy = 0;
		gridBagConstraints41.weightx = 1.0;
		gridBagConstraints41.gridwidth = 2;
		gridBagConstraints41.insets = STD_INSETS;
		GridBagConstraints gridBagConstraints52 = new GridBagConstraints();
		gridBagConstraints52.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints52.gridy = 11;
		gridBagConstraints52.weightx = 1.0;
		gridBagConstraints52.gridwidth = 2;
		gridBagConstraints52.insets = STD_INSETS;
		gridBagConstraints52.gridx = 2;
		
		// X: 3
		GridBagConstraints gridBagConstraints53 = new GridBagConstraints();
		gridBagConstraints53.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints53.gridy = 8;
		gridBagConstraints53.weightx = 1.0;
		gridBagConstraints53.insets = STD_INSETS;
		gridBagConstraints53.gridwidth = 1;
		gridBagConstraints53.gridx = 3;
		GridBagConstraints gridBagConstraints43 = new GridBagConstraints();
		gridBagConstraints43.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints43.gridy = 7;
		gridBagConstraints43.weightx = 1.0;
		gridBagConstraints43.insets = STD_INSETS;
		gridBagConstraints43.gridwidth = 2;
		gridBagConstraints43.gridx = 3;
		
		// X: 4
		GridBagConstraints gridBagConstraints62 = new GridBagConstraints();
		gridBagConstraints62.gridx = 4;
		gridBagConstraints62.gridy = 8;
		GridBagConstraints gridBagConstraints33 = new GridBagConstraints();
		gridBagConstraints33.gridx = 4;
		gridBagConstraints33.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints33.insets = STD_INSETS;
		gridBagConstraints33.gridy = 9;
		GridBagConstraints gridBagConstraints61 = new GridBagConstraints();
		gridBagConstraints61.gridx = 4;
		gridBagConstraints61.insets = STD_INSETS;
		gridBagConstraints61.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints61.gridy = 11;
		
		GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
		gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints7.gridx = 1;
		gridBagConstraints7.gridy = 2;
		gridBagConstraints7.weightx = 1.0;
		gridBagConstraints7.gridwidth = 4;
		gridBagConstraints7.anchor = GridBagConstraints.WEST;
		gridBagConstraints7.insets = STD_INSETS;
		
		GridBagConstraints gridBagConstraints24 = new GridBagConstraints();
		gridBagConstraints24.gridx = 1;
		gridBagConstraints24.insets = STD_INSETS;
		gridBagConstraints24.anchor = GridBagConstraints.WEST;
		gridBagConstraints24.gridwidth = 2;
		gridBagConstraints24.gridy = 7;
		GridBagConstraints gridBagConstraints34 = new GridBagConstraints();
		gridBagConstraints34.gridx = 1;
		gridBagConstraints34.insets = STD_INSETS;
		gridBagConstraints34.gridwidth = 2;
		gridBagConstraints34.anchor = GridBagConstraints.WEST;
		gridBagConstraints34.gridy = 8;
		
		GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
		gridBagConstraints21.gridx = 1;
		gridBagConstraints21.gridy = 0;
		GridBagConstraints gridBagConstraints42 = new GridBagConstraints();
		gridBagConstraints42.gridx = 1;
		gridBagConstraints42.gridy = 11;
		
		GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
		gridBagConstraints12.gridx = 1;
		gridBagConstraints12.gridwidth = 4;
		gridBagConstraints12.anchor = GridBagConstraints.WEST;
		gridBagConstraints12.gridy = 3;
		GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
		gridBagConstraints31.gridx = 1;
		gridBagConstraints31.gridwidth = 3;
		gridBagConstraints31.anchor = GridBagConstraints.WEST;
		gridBagConstraints31.gridy = 10;
		
		GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
		gridBagConstraints8.gridx = 4;
		gridBagConstraints8.insets = STD_INSETS;
		gridBagConstraints8.gridy = 0;
		
		GridBagConstraints gbcRLabels = new GridBagConstraints();
		gbcRLabels.gridx = 0;
		gbcRLabels.insets = STD_INSETS;
		gbcRLabels.anchor = GridBagConstraints.EAST;
		gbcRLabels.gridy = 0;
		
		jLUpdInterval = new JLabel("3.0 sec");
		jLUpdInterval.setHorizontalAlignment(SwingConstants.LEFT);
		jLUpdInterval.setMinimumSize(new Dimension(65, 15));
		jLUpdInterval.setMaximumSize(new Dimension(65, 15));
		jLUpdInterval.setPreferredSize(new Dimension(65, 15));
		jLPortStatus = new JLabel("");
		jLPortStatus.setPreferredSize(new Dimension(16, 16));
		jLHideLogDelay = new JLabel("");
		jLConnectionTimeout = new JLabel(Float.toString(((float)cfg.timeout / 1000F)) + " sec");
		
		this.setSize(449, 427);
		this.setLayout(new GridBagLayout());
		this.add(getJSUpdateInterval(), gridBagConstraints41);
		this.add(getJCAutoConnect(), gridBagConstraints6);
		this.add(getJCProtcol(), gridBagConstraints7);
		this.add(jLUpdInterval, gridBagConstraints8);
		this.add(getJCBUpdateInterval(), gridBagConstraints21);
		this.add(getJRLogShowAlway(), gridBagConstraints31);
		this.add(getJRLogHide(), gridBagConstraints42);
		this.add(getJSLogHideDelay(), gridBagConstraints52);
		this.add(jLHideLogDelay, gridBagConstraints61);
		this.add(getJSConnectionTimeout(), gridBagConstraints14);
		this.add(jLConnectionTimeout, gridBagConstraints33);
		this.add(new JLabel("Host:"), gridBagConstraints24);
		this.add(new JLabel("Port:"), gridBagConstraints34);
		this.add(getJTHost(), gridBagConstraints43);
		this.add(getJTPort(), gridBagConstraints53);
		this.add(jLPortStatus, gridBagConstraints62);
		
		addJComponents(new JComponent[] { new JLabel("Update Intervall:"), new JLabel("Automatische Verbindung:"), new JLabel("Verbindung:") }, gbcRLabels, true, this);
		gbcRLabels.gridy = 9;
		addJComponents(new JComponent[] { new JLabel("Verbindungs-Timeout:"), new JLabel("Log anzeigen:"), new JLabel("Log ausblenden:") }, gbcRLabels, true, this);
		addJComponents(new JComponent[] { getJCHttp11(), getJCChunkedTransfers(), getJCGzip(), getJCProxy() }, gridBagConstraints12, true, this);
		updateProxy();
	}
	
	private static void addJComponents(JComponent[] jc,
			GridBagConstraints gbc, boolean vertical, JPanel to) {
		
		if (jc == null || gbc == null || to == null) return;
		for (int i=0; i<jc.length; i++) {
			to.add(jc[i], gbc);
			if (vertical) gbc.gridy++; else gbc.gridx++;
		}
	}

	private JComboBox getJCAutoConnect() {
		if (jCAutoConnect == null) {
			jCAutoConnect = new JComboBox();
			jCAutoConnect.setPreferredSize(new Dimension(32, 18));
			jCAutoConnect.setMaximumSize(new Dimension(32767, 19));
			jCAutoConnect.setMinimumSize(new Dimension(32, 19));
			jCAutoConnect.addItem("Keine Verbindung");
			int x = 0;
			boolean found = false; 
			for (String d : cfg.connects.keySet()) {
				jCAutoConnect.addItem(d);
				if (d.equals(cfg.connectOnStartup)) found = true;
				if (!found) x++;
			}
			jCAutoConnect.setSelectedIndex((found) ? (++x) : (0));
			jCAutoConnect.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cfg.connectOnStartup = (jCAutoConnect.getSelectedItem().toString().equals("Keine Verbindung"))
							? ""
							: jCAutoConnect.getSelectedItem().toString();
				}
			});
		}
		return jCAutoConnect;
	}

	private JComboBox getJCProtcol() {
		if (jCProtcol == null) {
			jCProtcol = new JComboBox();
			jCProtcol.setMaximumSize(new Dimension(32767, 19));
			jCProtcol.setMinimumSize(new Dimension(32, 19));
			jCProtcol.addItem("http");
			jCProtcol.addItem("https");
			jCProtcol.setSelectedIndex((cfg.useHTTPS) ? (1) : (0));
			jCProtcol.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cfg.useHTTPS = jCProtcol.getSelectedItem().toString().equals("https");
				}
			});
		}
		return jCProtcol;
	}
	
	private JSlider getJSUpdateInterval() {
		if (jSUpdateInterval == null) {
			jSUpdateInterval = new JSlider();
			jSUpdateInterval.setMaximum(20);
			jSUpdateInterval.setValue(cfg.globalUpdateDelay / 500);
			jLUpdInterval.setText(Float.toString((float)jSUpdateInterval.getValue() / 2F) + " sec");
			jSUpdateInterval.addChangeListener(new javax.swing.event.ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					jLUpdInterval.setText((jSUpdateInterval.getValue() == 0)
							? ("aus")
							: Float.toString((float)jSUpdateInterval.getValue() / 2F) + " sec");
					cfg.globalUpdateDelay = jSUpdateInterval.getValue() * 500;
				}
			});
		}
		return jSUpdateInterval;
	}

	private JCheckBox getJCBUpdateInterval() {
		if (jCBUpdateInterval == null) {
			jCBUpdateInterval = new JCheckBox();
			jCBUpdateInterval.setSelected(cfg.useGlobalUpdateDelay);
			jSUpdateInterval.setEnabled(jCBUpdateInterval.isSelected());
			jLUpdInterval.setEnabled(jCBUpdateInterval.isSelected());
			jCBUpdateInterval.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cfg.useGlobalUpdateDelay = jCBUpdateInterval.isSelected();
					cfg.globalUpdateDelay = jSUpdateInterval.getValue() * 500;
					jSUpdateInterval.setEnabled(jCBUpdateInterval.isSelected());
					jLUpdInterval.setEnabled(jCBUpdateInterval.isSelected());
				}
			});
		}
		return jCBUpdateInterval;
	}

	private JRadioButton getJRLogShowAlway() {
		if (jRLogShowAlway == null) {
			JToggleButton.ToggleButtonModel toggleButtonModel = new JToggleButton.ToggleButtonModel();
			toggleButtonModel.setGroup(logShow);
			jRLogShowAlway = new JRadioButton();
			jRLogShowAlway.setText("immer");
			jRLogShowAlway.setModel(toggleButtonModel);
			jRLogShowAlway.setSelected(cfg.logShowAlways);
			jRLogShowAlway.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					cfg.logShowAlways = jRLogShowAlway.isSelected();
					jSLogHideDelay.setEnabled(!jRLogShowAlway.isSelected());
					jLHideLogDelay.setEnabled(!jRLogShowAlway.isSelected());
					cfg.log.show();
				}
			});
		}
		return jRLogShowAlway;
	}

	private JRadioButton getJRLogHide() {
		if (jRLogHide == null) {
			JToggleButton.ToggleButtonModel toggleButtonModel1 = new JToggleButton.ToggleButtonModel();
			toggleButtonModel1.setGroup(logShow);
			jRLogHide = new JRadioButton();
			jRLogHide.setModel(toggleButtonModel1);
			jRLogHide.setSelected(!cfg.logShowAlways);
			jRLogHide.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					cfg.logShowAlways = !jRLogHide.isSelected();
					if (jRLogHide.isSelected())
						cfg.log.closeAfterDelay();
				}
			});
		}
		return jRLogHide;
	}

	private JSlider getJSLogHideDelay() {
		if (jSLogHideDelay == null) {
			jSLogHideDelay = new JSlider();
			jSLogHideDelay.setValue(6);
			jSLogHideDelay.setMinimum(1);
			jSLogHideDelay.setMaximum(20);
			jSLogHideDelay.setValue(cfg.logHideDelay / 500);
			jLHideLogDelay.setText(Float.toString((float)cfg.logHideDelay / 1000F) + " sec");
			jSLogHideDelay.setEnabled(!jRLogShowAlway.isSelected());
			jLHideLogDelay.setEnabled(!jRLogShowAlway.isSelected());
			jSLogHideDelay.addChangeListener(new javax.swing.event.ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					jLHideLogDelay.setText(Float.toString((float)jSLogHideDelay.getValue() / 2F) + " sec");
					cfg.logHideDelay = jSLogHideDelay.getValue() * 500;
				}
			});
		}
		return jSLogHideDelay;
	}

	private JCheckBox getJCHttp11() {
		if (jCHttp11 == null) {
			jCHttp11 = new JCheckBox();
			jCHttp11.setText("HTTP 1.1 verwenden");
			jCHttp11.setSelected(cfg.useHTTP11);
			jCHttp11.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cfg.useHTTP11 = jCHttp11.isSelected();
				}
			});
		}
		return jCHttp11;
	}

	private JCheckBox getJCChunkedTransfers() {
		if (jCChunkedTransfers == null) {
			jCChunkedTransfers = new JCheckBox();
			jCChunkedTransfers.setText("Chunked Transfers verwenden");
			jCChunkedTransfers.setSelected(cfg.useChunkedTransfer);
			jCChunkedTransfers.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cfg.useChunkedTransfer = jCChunkedTransfers.isSelected();
				}
			});
		}
		return jCChunkedTransfers;
	}
	
	private JCheckBox getJCGzip() {
		if (jCGzip == null) {
			jCGzip = new JCheckBox();
			jCGzip.setText("GZip Kompression verwenden");
			jCGzip.setSelected(cfg.useGZipEncoding);
			jCGzip.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cfg.useGZipEncoding = jCGzip.isSelected();
				}
			});
		}
		return jCGzip;
	}

	private JSlider getJSConnectionTimeout() {
		if (jSConnectionTimeout == null) {
			jSConnectionTimeout = new JSlider();
			jSConnectionTimeout.setValue(100);
			jSConnectionTimeout.setMaximum(600);
			jSConnectionTimeout.setValue(Math.round(cfg.timeout / 100F));
			jSConnectionTimeout.addChangeListener(new javax.swing.event.ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					cfg.timeout = jSConnectionTimeout.getValue() * 100;
					jLConnectionTimeout.setText((jSConnectionTimeout.getValue() == 0)
							? ("unendlich")
							: ((float)jSConnectionTimeout.getValue() / 10F) + " sec");
				}
			});
		}
		return jSConnectionTimeout;
	}
	
	private JCheckBox getJCProxy() {
		if (jCProxy == null) {
			jCProxy = new JCheckBox();
			jCProxy.setSelected(cfg.isProxyEnabled());
			jCProxy.setText("Proxy benutzen");
			jCProxy.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					updateProxy();
				}
			});
			
		}
		return jCProxy;
	}
	
	private void updateProxy() {
		jTHost.setEnabled(jCProxy.isSelected());
		jTPort.setEnabled(jCProxy.isSelected());
		boolean ok = (
				jTPort.getText().matches("\\d+") &&
				Integer.valueOf(jTPort.getText()) > 0 &&
				jTHost.getText().length() > 0);
		cfg.setProxy((ok) ? (jTHost.getText()) : (null), (ok) ? (Integer.valueOf(jTPort.getText())) : (0));
		cfg.setProxyEnabled(jCProxy.isSelected());
		jLPortStatus.setIcon((ok || !jCProxy.isSelected()) ? (null) : (denied));
		jLPortStatus.setToolTipText((ok || !jCProxy.isSelected()) ? ("") : ("Port-Angabe ist ungültig"));
	}
	
	private JTextField getJTHost() {
		if (jTHost == null) {
			jTHost = new JTextField();
			jTHost.setText(cfg.getProxyHost());
			jTHost.addCaretListener(new javax.swing.event.CaretListener() {
				public void caretUpdate(javax.swing.event.CaretEvent e) {
					updateProxy();
				}
			});
		}
		return jTHost;
	}
	
	private JTextField getJTPort() {
		if (jTPort == null) {
			jTPort = new JTextField();
			jTPort.setText(Integer.toString(cfg.getProxyPort()));
			jTPort.addCaretListener(new javax.swing.event.CaretListener() {
				public void caretUpdate(javax.swing.event.CaretEvent e) {
					updateProxy();
				}
			});
		}
		return jTPort;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
