/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package yacy.datatypes;

import java.util.ArrayList;

public class UrlInfo {
	
	public static final int HTML_ESCAPED = 1;
	public static final int HTML_NORMAL = 2;
	public static final int SINGLE_SENTENCES = 3;
	
	private String 	url 		= null;
	private String 	urlHash 	= null;
	private int 	wc 			= 0;
	private String 	desc 		= null;
	private long 	size 		= 0;
	private String 	mimeType 	= null;
	private int 	statusCode 	= 0;
	private String 	statusText 	= null;
	private int 	mode 		= 0;
	private ArrayList<String> sentences = null;
	
	public UrlInfo(
			String url,
			String urlHash,
			int wc,
			String desc,
			long size,
			String mimeType,
			int statusCode,
			String statusText,
			int mode,
			String body) {
		this.url = url;
		this.urlHash = urlHash;
		this.wc = wc;
		this.desc = desc;
		this.size = size;
		this.mimeType = mimeType;
		this.statusCode = statusCode;
		this.statusText = statusText;
		this.mode = mode;
		if (mode == HTML_ESCAPED || mode == HTML_NORMAL)
			for (String d : body.split("$"))
				this.sentences.add(d);
	}
	
	public UrlInfo(
			String url,
			String urlHash,
			int wc,
			String desc,
			long size,
			String mimeType,
			int statusCode,
			String statusText,
			int mode,
			ArrayList<String> sentences) {
		this.url = url;
		this.urlHash = urlHash;
		this.wc = wc;
		this.desc = desc;
		this.size = size;
		this.mimeType = mimeType;
		this.statusCode = statusCode;
		this.statusText = statusText;
		this.mode = mode;
		this.sentences = sentences;
	}
	
	public String 	getDescription() 		{ return this.desc; }
	public String 	getMimeType() 			{ return this.mimeType; }
	public int 		getMode() 				{ return this.mode; }
	public ArrayList<String> getSentences() { return this.sentences; }
	public long 	getSize() 				{ return this.size; }
	public int 		getStatusCode() 		{ return this.statusCode; }
	public String 	getStatusText() 		{ return this.statusText; }
	public String 	getURL() 				{ return this.url; }
	public String 	getURLHash() 			{ return this.urlHash; }
	public int 		getWordCount() 			{ return this.wc; }
}
